@@ -2,6 +2,7 @@ package controllers
 
 import (
 	"context"
+	"encoding/json"
 	"fmt"
 	"net"
 	"strconv"
@@ -9,12 +10,14 @@ import (
 	"time"
 
 	"github.com/go-logr/logr"
+	clusterv1alpha1 "github.com/open-cluster-management/api/cluster/v1alpha1"
 	clusterv1beta1 "github.com/open-cluster-management/multicloud-operators-foundation/pkg/apis/internal.open-cluster-management.io/v1beta1"
 	"github.com/open-cluster-management/multicloud-operators-foundation/pkg/klusterlet/agent"
 	routev1 "github.com/openshift/client-go/route/clientset/versioned"
 	"github.com/prometheus/common/log"
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/equality"
+	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
@@ -37,6 +40,7 @@ type ClusterInfoReconciler struct {
 	KubeClient                  kubernetes.Interface
 	ManagedClusterDynamicClient dynamic.Interface
 	RouteV1Client               routev1.Interface
+	ClusterName                 string
 	MasterAddresses             string
 	AgentAddress                string
 	AgentIngress                string
@@ -337,6 +341,8 @@ func (r *ClusterInfoReconciler) getVendor(gitVersion string, isOpenShift bool) (
 		return
 	case strings.Contains(gitVersion, string(clusterv1beta1.KubeVendorICP)):
 		kubeVendor = clusterv1beta1.KubeVendorICP
+	case r.isOpenshiftDedicated():
+		kubeVendor = clusterv1beta1.KubeVendorOSD
 	case isOpenShift:
 		kubeVendor = clusterv1beta1.KubeVendorOpenShift
 	default:
@@ -373,6 +379,10 @@ func (r *ClusterInfoReconciler) getCloudVendor() clusterv1beta1.CloudVendorType
 		return clusterv1beta1.CloudVendorAWS
 	case strings.Contains(providerID, "gce"):
 		return clusterv1beta1.CloudVendorGoogle
+	case strings.Contains(providerID, "vsphere"):
+		return clusterv1beta1.CloudVendorVSphere
+	case strings.Contains(providerID, "openstack"):
+		return clusterv1beta1.CloudVendorOpenStack
 	}
 
 	return clusterv1beta1.CloudVendorOther
@@ -459,18 +469,48 @@ func (r *ClusterInfoReconciler) isOpenshift() bool {
 	return false
 }
 
+func (r *ClusterInfoReconciler) isOpenshiftDedicated() bool {
+	hasProject := false
+	hasManaged := false
+	serverGroups, err := r.KubeClient.Discovery().ServerGroups()
+	if err != nil {
+		klog.Errorf("failed to get server group %v", err)
+		return false
+	}
+	for _, apiGroup := range serverGroups.Groups {
+		if apiGroup.Name == "project.openshift.io" {
+			hasProject = true
+		}
+		// this API group is created for the Openshift Dedicated platform to manage various permissions.
+		// defined in https://github.com/openshift/rbac-permissions-operator/blob/master/pkg/apis/managed/v1alpha1/subjectpermission_types.go
+		if apiGroup.Name == "managed.openshift.io" {
+			hasManaged = true
+		}
+	}
+	return hasProject && hasManaged
+}
+
 var ocpVersionGVR = schema.GroupVersionResource{
 	Group:    "config.openshift.io",
 	Version:  "v1",
 	Resource: "clusterversions",
 }
 
+const (
+	OCP3Version = "3"
+)
+
 func (r *ClusterInfoReconciler) getOCPDistributionInfo() (clusterv1beta1.OCPDistributionInfo, string, error) {
 	ocpDistributionInfo := clusterv1beta1.OCPDistributionInfo{}
 	obj, err := r.ManagedClusterDynamicClient.Resource(ocpVersionGVR).Get(context.TODO(), "version", metav1.GetOptions{})
 	if err != nil {
+		if apierrors.IsNotFound(err) {
+			ocpDistributionInfo.DesiredVersion = OCP3Version
+			ocpDistributionInfo.Version = OCP3Version
+			return ocpDistributionInfo, "", nil
+		}
 		klog.Errorf("failed to get OCP cluster version: %v", err)
-		return ocpDistributionInfo, "", err
+		return ocpDistributionInfo, "", client.IgnoreNotFound(err)
 	}
 
 	clusterID, _, err := unstructured.NestedString(obj.Object, "spec", "clusterID")
@@ -532,7 +572,7 @@ func (r *ClusterInfoReconciler) getOCPDistributionInfo() (clusterv1beta1.OCPDist
 				klog.Errorf("failed to get OCP cluster version in latest history of status: %v", err)
 				continue
 			}
-			if conditionstatus == "True" {
+			if conditionstatus == "True" && ocpDistributionInfo.DesiredVersion != ocpDistributionInfo.Version {
 				ocpDistributionInfo.UpgradeFailed = true
 			}
 			break
@@ -559,6 +599,87 @@ func (r *ClusterInfoReconciler) getDistributionInfoAndClusterID() (clusterv1beta
 	return distributionInfo, clusterID, nil
 }
 
+var ocpInfrGVR = schema.GroupVersionResource{
+	Group:    "config.openshift.io",
+	Version:  "v1",
+	Resource: "infrastructures",
+}
+
+// should be the type defined in infrastructure.config.openshift.io
+const (
+	AWSPlatformType = "AWS"
+	GCPPlatformType = "GCP"
+	IBMPlatformType = "IBM"
+)
+
+func (r *ClusterInfoReconciler) getClusterRegion() string {
+	var region = ""
+
+	switch {
+	case r.isOpenshift():
+		obj, err := r.ManagedClusterDynamicClient.Resource(ocpInfrGVR).Get(context.TODO(), "cluster", metav1.GetOptions{})
+		if err != nil {
+			klog.Errorf("failed to get OCP infrastructures.config.openshift.io/cluster: %v", err)
+			return ""
+		}
+
+		platformType, _, err := unstructured.NestedString(obj.Object, "status", "platform")
+		if err != nil {
+			klog.Errorf("failed to get OCP platform type in status of infrastructures.config.openshift.io/cluster: %v", err)
+			return ""
+		}
+
+		// only ocp on aws and gcp has region definition
+		// refer to https://github.com/openshift/api/blob/master/config/v1/types_infrastructure.go
+		switch platformType {
+		case AWSPlatformType:
+			region, _, err = unstructured.NestedString(obj.Object, "status", "platformStatus", "aws", "region")
+			if err != nil {
+				klog.Errorf("failed to get OCP region in status of infrastructures.config.openshift.io/cluster: %v", err)
+				return ""
+			}
+		case GCPPlatformType:
+			region, _, err = unstructured.NestedString(obj.Object, "status", "platformStatus", "gcp", "region")
+			if err != nil {
+				klog.Errorf("failed to get OCP region in status of infrastructures.config.openshift.io/cluster: %v", err)
+				return ""
+			}
+		}
+	}
+
+	return region
+}
+
+type InfraConfig struct {
+	InfraName string `json:"infraName,omitempty"`
+}
+
+func (r *ClusterInfoReconciler) getInfraConfig() string {
+	if !r.isOpenshift() {
+		return ""
+	}
+
+	obj, err := r.ManagedClusterDynamicClient.Resource(ocpInfrGVR).Get(context.TODO(), "cluster", metav1.GetOptions{})
+	if err != nil {
+		klog.Errorf("failed to get OCP infrastructures.config.openshift.io/cluster: %v", err)
+		return ""
+	}
+
+	infraConfig := InfraConfig{}
+	infraConfig.InfraName, _, err = unstructured.NestedString(obj.Object, "status", "infrastructureName")
+	if err != nil {
+		klog.Errorf("failed to get OCP infrastructure Name in status of infrastructures.config.openshift.io/cluster: %v", err)
+		return ""
+	}
+
+	infraConfigRaw, err := json.Marshal(infraConfig)
+	if err != nil {
+		klog.Errorf("failed to marshal infraConfig: %v", err)
+		return ""
+	}
+	return string(infraConfigRaw)
+}
+
 func (r *ClusterInfoReconciler) RefreshAgentServer(clusterInfo *clusterv1beta1.ManagedClusterInfo) {
 	select {
 	case r.Agent.RunServer <- *clusterInfo:
@@ -574,3 +695,115 @@ func (r *ClusterInfoReconciler) SetupWithManager(mgr ctrl.Manager) error {
 		For(&clusterv1beta1.ManagedClusterInfo{}).
 		Complete(r)
 }
+
+func (r *ClusterInfoReconciler) GetClusterClaims() ([]*clusterv1alpha1.ClusterClaim, error) {
+	claims := []*clusterv1alpha1.ClusterClaim{}
+	distributionInfo, clusterID, err := r.getDistributionInfoAndClusterID()
+	if err != nil {
+		return nil, fmt.Errorf("failed to get distribution info and clusterID, error: %w ", err)
+	}
+
+	// handle OpenShift specific claims
+	if len(clusterID) > 0 {
+		claims = append(claims, newClusterClaim("id.openshift.io", clusterID))
+	}
+	if len(distributionInfo.OCP.Version) > 0 {
+		claims = append(claims, newClusterClaim("version.openshift.io", distributionInfo.OCP.Version))
+	}
+
+	infraConfig := r.getInfraConfig()
+	if len(infraConfig) > 0 {
+		claims = append(claims, newClusterClaim("infrastructure.openshift.io", infraConfig))
+	}
+
+	_, _, consoleURL := r.getMasterAddresses()
+	if len(consoleURL) > 0 {
+		claims = append(claims, newClusterClaim("consoleurl.cluster.open-cluster-management.io", consoleURL))
+	}
+
+	// handle reserved claims
+	claims = append(claims, newClusterClaim("id.k8s.io", r.ClusterName))
+	kubeVersion, platform, product := r.getVersionPlatformProduct(distributionInfo)
+	claims = append(claims, newClusterClaim("kubeversion.open-cluster-management.io", kubeVersion))
+	if len(platform) > 0 {
+		claims = append(claims, newClusterClaim("platform.open-cluster-management.io", platform))
+	}
+	if len(product) > 0 {
+		claims = append(claims, newClusterClaim("product.open-cluster-management.io", product))
+	}
+
+	region := r.getClusterRegion()
+	if len(region) > 0 {
+		claims = append(claims, newClusterClaim("region.open-cluster-management.io", region))
+	}
+
+	return claims, nil
+}
+
+func (r *ClusterInfoReconciler) getVersionPlatformProduct(distributionInfo clusterv1beta1.DistributionInfo) (kubeVersion, platform, product string) {
+	kubeVersion = r.getVersion()
+	isOpenShift := distributionInfo.Type == clusterv1beta1.DistributionTypeOCP
+	cloudVendor := r.getCloudVendor()
+	gitVersion := strings.ToUpper(kubeVersion)
+
+	// deal with product and platform
+	switch {
+	case strings.Contains(gitVersion, string(clusterv1beta1.KubeVendorIKS)):
+		// IBM Cloud Platform + IKS
+		platform = IBMPlatformType
+		product = string(clusterv1beta1.KubeVendorIKS)
+		return
+	case strings.Contains(gitVersion, string(clusterv1beta1.KubeVendorEKS)):
+		// AWS + EKS
+		platform = AWSPlatformType
+		product = string(clusterv1beta1.KubeVendorEKS)
+		return
+	case strings.Contains(gitVersion, string(clusterv1beta1.KubeVendorGKE)):
+		// Google Cloud Platform + GKE
+		platform = GCPPlatformType
+		product = string(clusterv1beta1.KubeVendorGKE)
+		return
+	case r.isOpenshiftDedicated():
+		product = string(clusterv1beta1.KubeVendorOSD)
+	case isOpenShift:
+		product = string(clusterv1beta1.KubeVendorOpenShift)
+	}
+
+	switch cloudVendor {
+	case clusterv1beta1.CloudVendorIBM:
+		// IBM Cloud Platform
+		platform = IBMPlatformType
+	case clusterv1beta1.CloudVendorAWS:
+		// AWS
+		platform = AWSPlatformType
+	case clusterv1beta1.CloudVendorGoogle:
+		// Google Cloud Platform
+		platform = GCPPlatformType
+	case clusterv1beta1.CloudVendorAzure:
+		// Azure
+		platform = string(clusterv1beta1.CloudVendorAzure)
+		if !isOpenShift {
+			// Azure + AKS
+			product = string(clusterv1beta1.KubeVendorAKS)
+		}
+	case clusterv1beta1.CloudVendorOpenStack:
+		// OpenStack
+		platform = string(clusterv1beta1.CloudVendorOpenStack)
+	case clusterv1beta1.CloudVendorVSphere:
+		// VSphere
+		platform = string(clusterv1beta1.CloudVendorVSphere)
+	}
+
+	return kubeVersion, platform, product
+}
+
+func newClusterClaim(name, value string) *clusterv1alpha1.ClusterClaim {
+	return &clusterv1alpha1.ClusterClaim{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: name,
+		},
+		Spec: clusterv1alpha1.ClusterClaimSpec{
+			Value: value,
+		},
+	}
+}
