@@ -18,6 +18,7 @@ import (
 	clusterv1client "github.com/open-cluster-management/api/client/cluster/clientset/versioned"
 	clusterv1 "github.com/open-cluster-management/api/cluster/v1"
 	clusterinfov1beta1 "github.com/open-cluster-management/multicloud-operators-foundation/pkg/apis/internal.open-cluster-management.io/v1beta1"
+	hiveclient "github.com/openshift/hive/pkg/client/clientset/versioned"
 
 	certificatesv1beta1 "k8s.io/api/certificates/v1beta1"
 	"k8s.io/apimachinery/pkg/api/errors"
@@ -90,6 +91,20 @@ func NewKubeClient() (kubernetes.Interface, error) {
 	return kubernetes.NewForConfig(cfg)
 }
 
+func NewHiveClient() (hiveclient.Interface, error) {
+	kubeConfigFile, err := getKubeConfigFile()
+	if err != nil {
+		return nil, err
+	}
+
+	cfg, err := clientcmd.BuildConfigFromFlags("", kubeConfigFile)
+	if err != nil {
+		return nil, err
+	}
+
+	return hiveclient.NewForConfig(cfg)
+}
+
 func NewAPIServiceClient() (*apiregistrationclient.ApiregistrationV1Client, error) {
 	kubeConfigFile, err := getKubeConfigFile()
 	if err != nil {
@@ -259,6 +274,22 @@ func UpdateClusterResource(dynamicClient dynamic.Interface,
 	return dynamicClient.Resource(gvr).Update(context.TODO(), obj, metav1.UpdateOptions{})
 }
 
+func UpdateResource(dynamicClient dynamic.Interface,
+	gvr schema.GroupVersionResource,
+	obj *unstructured.Unstructured) (*unstructured.Unstructured, error) {
+	var rs *unstructured.Unstructured
+	err := retry.RetryOnConflict(retry.DefaultBackoff, func() error {
+		oldObj, err := GetResource(dynamicClient, gvr, obj.GetNamespace(), obj.GetName())
+		if err != nil {
+			return err
+		}
+		obj.SetResourceVersion(oldObj.GetResourceVersion())
+		rs, err = dynamicClient.Resource(gvr).Namespace(obj.GetNamespace()).Update(context.TODO(), obj, metav1.UpdateOptions{})
+		return err
+	})
+	return rs, err
+}
+
 func CreateResource(dynamicClient dynamic.Interface, gvr schema.GroupVersionResource, obj *unstructured.Unstructured) (*unstructured.Unstructured, error) {
 	return dynamicClient.Resource(gvr).Namespace(obj.GetNamespace()).Create(context.TODO(), obj, metav1.CreateOptions{})
 }
@@ -679,7 +710,7 @@ func CheckFoundationPodsReady() error {
 	}
 
 	if pods, err := hubClient.CoreV1().Pods("open-cluster-management-agent").List(context.TODO(),
-		metav1.ListOptions{LabelSelector: "app=foundation-agent"}); err != nil {
+		metav1.ListOptions{LabelSelector: "app=work-manager"}); err != nil {
 		if len(pods.Items) == 0 {
 			return fmt.Errorf("failed to get agent pods")
 		}
